package org.cfpm.ruth.modifiedCloud;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.Enumeration;
import java.util.Hashtable;
import java.lang.Math;
/**
 * Title: Set up the actual climate
 * Description: generates weather in three terciles as wet (above normal rainfall)
 * normal or dry (below normal)
 * WMO definition of normal is the 30 year average
 * Copyright: Copyright (c) 2004
 * Company: University of Cambridge/SEI Oxford
 * @author Mike Bithell/SukainaBharwani
 * @version 1.0
 */
public class Climate {
  public static final int totalMonths=1680;
  public static final int totalYears=140;
  public static float rainfall[] = new float[totalMonths];
  public static float N[] = new float[totalYears];
  public static float OND[] = new float[totalYears];
  public static int climate_history[] = new int[totalYears];
  public boolean expect_dry = false;
  public int user_defined_normal;
  public Weather currentWeather = new Weather();
  public int time;
  private float wetDeviation=0,dryDeviation=0;
  /**
   * At initialisation, all rainfall data is read in and stored in an array. Threshold values for AN,
   * BN and N are also read and stored in the same way, for use when the values for rainfall
   * need to be converted to normal, wet and dry.
   */
  public Climate() {
    try {
      String line;
      float number;
      int count=1;

      BufferedReader input = new BufferedReader(new FileReader("prec.txt"));
      line=input.readLine();
      while (line!=null){
        number=Float.parseFloat(line);
        rainfall[count] = number;
        count++;
        line=input.readLine();
      }
      for (int i = 0; i < rainfall.length; i++) {
     //    System.out.println(rainfall[i]);
     }
   }
    catch (Exception e) {
      System.out.println(e.getMessage());
    }
  }

  /**
   * Calls method to convert the rainfall data for the current year to normal, wet or dry given
   * current thresholds which are retrieved from the relevant files.
   * @param year start of season
   */
  public void update(int year) {
    generateWeather(year); //start of the season
  }

  /**
   * Original distribution is normal 40% of the time, wet 20% of the time and dry 40% of the time.
   * As real rainfall data is now being used, this may not be exact. In order to utilise real climate
   * data, real rainfall values will need to be converted to a value representing normal, wet or dry,
   * depending on relevant thresholds. This could be based meteorologically or empirically based on
   * fieldwork and perceptions of what is normal, wet and dry. Both will mean important differences
   * in the strategies chosen since what is considered normal by some farmers is not the case meteorologically.
   * @param year current year
   */
  public void generateWeather(int year) {
    currentWeather.setRaintype(Weather.NORMAL);
    climate_history[year]=Weather.NORMAL;
    //if the seasonal mean is greater than the 30 year average
    //then its wet
    if (OND[year] >= N[year]+wetDeviation) {
      currentWeather.setRaintype(Weather.WET);
      climate_history[year]=Weather.WET;
    }
    //otherwise its dry.
    if (OND[year] <= N[year]+dryDeviation) {
      currentWeather.setRaintype(Weather.DRY);
      climate_history[year] = Weather.DRY;
    }
 //   System.out.println(year+ ". "+climate_history[year]);
  }

  /**
   * Generates meterological normal threshold which is calculated as a 30 year average.
   * This can be varied by the user also so that experiments can be conducted varying the met normal
   * to correlate more with the percieved normal.
   */
  public void generateNThreshold() {

    //find October-November seasonal mean for all the years in the dataset
    //note arrays start at 0
    for (int year = 0; year < totalYears; year++) { // loop for 140 years / 1680 months
      float sum=0;
      for (int month=(year*12+FarmingWorld.OCTOBER-FarmingWorld.JANUARY);month<(year*12+FarmingWorld.OCTOBER-FarmingWorld.JANUARY+3);month++){
        sum = sum + rainfall[month];
      }
      OND[year]=sum/3;
    }
    //Now find the year running mean to get "Normal" conditions
    //normal of 30 years matches the WMO definition
    float av=0;
    for (int year=0;year<getNormalYears();year++){
      av+=OND[year];
    }
    //The first set of years are set to the same value
    //This isn't strictly correct - we really need a set of  years
    //rain prior to the start date to set this
    for (int year=0;year<getNormalYears();year++)N[year]=av/getNormalYears();
    //for the remainder we calculate the running mean of the previous set
    //of years
    for (int year=getNormalYears();year<totalYears;year++){
      N[year]=N[year-1]+(OND[year]-OND[year-getNormalYears()])/getNormalYears();
    }
    //now find out how much the seasonal means differ from the
    //long term average - normal rain will be defined
    float max=-1,min=10000,dev=0;
    for (int year=getNormalYears();year<totalYears;year++){
      dev=OND[year]-N[year];
      if (dev>max)max=dev;
      if (dev<min)min=dev;
    }
 //   System.out.println("maxmin "+max+" "+min);
    //now we have range of deviations from the mean - calculate the
    //histogram and cumulative distribution in 200 bins
    float range=max-min;
    float[] hist=new float[200],cum=new float[200];
    for (int year=getNormalYears();year<totalYears;year++){
      dev=OND[year]-N[year];
      int thresh=(int)((dev-min)/range*199);
      hist[thresh]++;
    }
    //now find the terciles of the distribution
    cum[0]=hist[0];int tdry=0,twet=0;
    for (int i=1;i<200;i++){
      cum[i]=hist[i]+cum[i-1];
      if (cum[i]<=totalYears/3)tdry=i;
      if (cum[i]<=totalYears*2./3)twet=i;
    }
    wetDeviation=(float)twet/200*range+min;
    dryDeviation=(float)tdry/200*range+min;
//    System.out.println("wetdry "+wetDeviation+" "+dryDeviation);
  }

  /**
   *
   * @param n from World class, a user-defined parameter which allows met normal to be varied.
   */
  public void setNormalYears(int n){
    user_defined_normal=n;
  }

  /**
   *
   * @return normal value defined by the user in the RePast parameter pane
   */
  public int getNormalYears(){
    return user_defined_normal;
  }

  /**
   * Need to provide markers for drought and flood events, so that they provide an anchor
   * for memory and bias in future response.
   * @param year current year
   * @return array object containing climate history
   */
 public int getHistory(int year) {
   return climate_history[year];
  }

  /**
   * Influence of past experience on perception and expectation of foreacst information
   * e.g. if more dry years than wet in memory, expect drier forecast. Eg. normal if forecast is for wet.
   * @param current_year current year
   * @param years number of years in memory
   * @return bias whether towards expectation of wetter or drier weather.
   */
  public boolean calcClimateMemory(int current_year, int years){
    int wet=0, dry=0;
    for (int i=current_year; i > current_year - years; i--) {
        if (getHistory(i-1) == Weather.WET) {
          wet++;
        }
        else {
          dry++;
        }
    }
    if (dry > wet) {
      expect_dry = true;
     //if more dry years than wet in memory, expect drier forecast. Eg. normal if forecast is for wet.
    }
    else {
      expect_dry = false;
    }return expect_dry;
  }
}


